Vaultx.Auth.TokenRenewal (Vaultx v0.7.0)

Background token renewal worker.

Periodically checks the current client token TTL and renews it when the remaining TTL percentage drops below the configured threshold.

This worker is started conditionally by the application when:

  • token_renewal_enabled is true
  • a token is configured
  • and we are not running in the test environment

It uses Vaultx.Auth.Token.lookup_self/1 and renew_token/2 which already emit Telemetry and Audit events, so this worker only orchestrates timing.

Summary

Functions

Returns a specification to start this module under a supervisor.

Functions

child_spec(init_arg)

Returns a specification to start this module under a supervisor.

See Supervisor.

start_link(opts)