Vaultx.Cache.Manager (Vaultx v0.7.0)
View SourceCache manager that coordinates operations across multiple cache layers.
This GenServer manages the lifecycle and coordination of all cache layers, handles cache warming, cleanup, and provides a unified interface for cache operations.
Summary
Functions
Returns a specification to start this module under a supervisor.
Clears cache layers.
Deletes a value from all cache layers.
Gets a value from the cache layers.
Gets multiple values from cache.
Puts a value into the cache layers.
Puts multiple key-value pairs.
Starts the cache manager.
Warms the cache with preloaded data.
Functions
Returns a specification to start this module under a supervisor.
See Supervisor.
Clears cache layers.
Deletes a value from all cache layers.
Gets a value from the cache layers.
Gets multiple values from cache.
Puts a value into the cache layers.
Puts multiple key-value pairs.
Starts the cache manager.
Warms the cache with preloaded data.